package de.fuberlin.wiwiss.d2rq.dblp2jena;

import java.util.Iterator;

import com.hp.hpl.jena.db.DBConnection;
import com.hp.hpl.jena.db.IDBConnection;
import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;

import de.fuberlin.wiwiss.d2rq.D2RQException;
import de.fuberlin.wiwiss.d2rq.GraphD2RQ;

/**
 * Runs a find(s,p,o) query against a database-backed Jena model
 * and a D2RQ-mapped SQL database. You have to provide the filename
 * of the D2RQ map and the database connection for Jena in the constants
 * below. The database connection for D2RQ is in the map file.
 * 
 * To run different queries, you have to change the s, p and o
 * constants below. You'll find many example queries in the comments
 * below.
 * 
 * @author Richard Cyganiak <richard@cyganiak.de>
 * @version $Id$
 */
public class Compare {
	static final String D2RQ_MAP = "map/dblp-d2rq.n3";
	static String M_DB_URL = "jdbc:mysql:///dblp_jena"; // database URL
	static String M_DB_USER = "root";     // database user id
	static String M_DB_PASSWD = "";   // database user password
	static String M_DB = "MySQL";  // database engine
	static String M_DBDRIVER_CLASS = "com.mysql.jdbc.Driver";  // database driver

// ===== Some Queries ======
	
	static final int times = 500;
	static final Node s = Node.createURI("http://dblp.uni-trier.de/inproceeding/16655");
	static final Node p = Node.ANY;
	static final Node o = Node.ANY;

//	static final int times = 500;
//	static final Node s = Node.createURI("http://dblp.uni-trier.de/proceeding/160");
//	static final Node p = Node.ANY;
//	static final Node o = Node.ANY;

//	static final int times = 500;
//	static final Node s = Node.createURI("http://nope.example.net/");
//	static final Node p = Node.ANY;
//	static final Node o = Node.ANY;

//	static final int times = 500;
//	static final Node s = Node.ANY;
//	static final Node p = Node.createURI("http://dblp.uni-trier.de/xml/dplp.dtd/personName");
//	static final Node o = Node.createLiteral("M. Rieskamp", null, XSDDatatype.XSDstring);

//	static final int times = 500;
//	static final Node s = Node.ANY;
//	static final Node p = Node.createURI("http://dblp.uni-trier.de/xml/dplp.dtd/inProceedingsAuthor");
//	static final Node o = Node.createURI("http://dblp.uni-trier.de/person/5");

//	static final int times = 500;
//	static final Node s = Node.ANY;
//	static final Node p = Node.createURI("http://dblp.uni-trier.de/xml/dplp.dtd/seriesTitle");
//	static final Node o = Node.createLiteral("Topics in Information Systems", null, XSDDatatype.XSDstring);

//	static final int times = 1;
//	static final Node s = Node.ANY;
//	static final Node p = Node.createURI("http://dblp.uni-trier.de/xml/dplp.dtd/seriesTitle");
//	static final Node o = Node.ANY;
	
//	static final int times = 500;
//	static final Node s = Node.ANY;
//	static final Node p = Node.ANY;
//	static final Node o = Node.createURI("http://dblp.uni-trier.de/proceeding/12");

//	static final int times = 1;
//	static final Node s = Node.ANY;
//	static final Node p = Node.ANY;
//	static final Node o = Node.createLiteral("Cat Photo Collection", null, XSDDatatype.XSDstring);

//	static final int times = 1;
//	static final Node s = Node.ANY;
//	static final Node p = Node.ANY;
//	static final Node o = Node.ANY;

//	===== End of Some Queries ======
	
	public static void main(String[] args) throws D2RQException, ClassNotFoundException {

		Class.forName(M_DBDRIVER_CLASS);
		IDBConnection conn = new DBConnection(M_DB_URL, M_DB_USER, M_DB_PASSWD, M_DB);
		Model jenaModel = ModelFactory.createModelRDBMaker(conn).createModel("dblp");
		jenaModel.contains(jenaModel.getResource("http://dblp.uni-trier.de/inProceeding/16655"), jenaModel.getProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type"), jenaModel.getResource("http://dblp.uni-trier.de/xml/dplp.dtd/InProceeding"));
		Graph rdbGraph = jenaModel.getGraph();

		FindTimer jenaTimer = new FindTimer(rdbGraph, s, p, o, times);
		jenaTimer.go();
		long jenaTime = jenaTimer.getTime();
		long jenaResultCount = jenaTimer.getResultCount();
		rdbGraph.close();
		
//		D2RQ.debug = true;
		GraphD2RQ graphD2RQ = new GraphD2RQ(D2RQ_MAP);
		FindTimer D2RQTimer = new FindTimer(graphD2RQ, s, p, o, times);
		D2RQTimer.go();
		long D2RQTime = D2RQTimer.getTime();
		long D2RQResultCount = D2RQTimer.getResultCount();
		graphD2RQ.close();

		System.out.println("find(s p o): " + s + " " + p + " " + o);
		System.out.println("(run " + times + " times)");
		System.out.println("Jena: " + jenaTime + " ms");
		System.out.println("D2RQ: " + D2RQTime + " ms");
		if (jenaResultCount == D2RQResultCount) {
			System.out.println("                    (" + jenaResultCount + " results)");
		} else {
			System.out.println("!!! RESULT COUNT MISMATCH: Jena " + jenaResultCount + " vs. D2RQ " + D2RQResultCount);
			System.out.println("===== Jena =====");
			Iterator iter = jenaTimer.getResults().iterator();
			while (iter.hasNext()) {
				System.out.println(iter.next());
			}
			System.out.println("===== D2RQ =====");
			iter = D2RQTimer.getResults().iterator();
			while (iter.hasNext()) {
				System.out.println(iter.next());
			}
		}
	}
}
